#include "api/df_lists.h"
#include "datafox_library_iv.h"
#include <cstring>

DF_API_EXTERN int DF_API_CALL DFCTableOpen( int connectionId, int deviceAddress, const char *name, int options, int *tableHandle )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( channelBridge->isValidDeviceAddress( deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false
        || name == nullptr || tableHandle == nullptr || options != 0 )
    {
        if ( channelBridge->isValidDeviceAddress( deviceAddress ) == false )
        {
            deviceAddress = 0;
        }

        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER );

        channelBridge->logEnd( "\t%s\t%i\t%i\t(%i, %i)", (name == nullptr) ? "" : name, options, (tableHandle == nullptr) ? 0 : *tableHandle, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED );
        channelBridge->logEnd( "\t%s\t%i\t%i\t(%i, %i)", name, options, *tableHandle, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    std::string reserved;
    int iRes = channelBridge->tableOpen( deviceAddress, name, options, reinterpret_cast<int &>(*tableHandle), reserved );
    if ( iRes > IV_ERROR_SUCCESS )
    { 
        int errorNumber = IV_ERROR_SUCCESS;
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ( (iRes >> 8) & 0xFF )
        {
            if ( ((iRes >> 8) & 0xFF) == 2 )
            {
                errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch ( iRes & 0xFF )
            {
            case 1:
            case 2:
                errorNumber = IV_ERROR_NAME_UNKNOWN;
                break;

            case 28:
                errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                errorNumber = IV_ERROR_PROCESSING;
            }
        }

        channelBridge->setLastErrorNumber( deviceAddress, errorNumber );
        iRes = 0;
    }
    else if ( iRes < IV_ERROR_SUCCESS )
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber( deviceAddress, -iRes );
        iRes = 0;
    }
    else
    {
        iRes = 1;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if ( iRes == 0 )
    {
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%s\t%i\t%i\t(%i, %i)", name, options, *tableHandle, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCOpenTable(int connectionId, int deviceAddress, const char *name, int options, int *tableHandle, const char *reserved, int *errorNumber)
{
    int res = 1;
    if (errorNumber == nullptr)
        return 0;

    if (DFCTableOpen(connectionId, deviceAddress, name, options, tableHandle) == false)
    {
        ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
        // Wurde Channel-Objekt ermittelt.
        if (channelBridge == nullptr)
            return 0;

        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);
        res = 0;
    }

    return res;
}

DF_API_EXTERN int DF_API_CALL DFCTableClose(int connectionId, int deviceAddress, int tableHandle)
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, tableHandle);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        channelBridge->logEnd("\t(%i, %i)", 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        channelBridge->logEnd("\t(%i, %i)", 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iRes = channelBridge->tableClose(deviceAddress, tableHandle);
    if (iRes > IV_ERROR_SUCCESS)
    {
        int errorNumber = IV_ERROR_SUCCESS;
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                errorNumber = IV_ERROR_HANDLE;
                break;

            case 28:
                errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                errorNumber = IV_ERROR_PROCESSING;
            }
        }

        channelBridge->setLastErrorNumber(deviceAddress, errorNumber);
        iRes = 0;
    }
    else if (iRes < IV_ERROR_SUCCESS)
    { 
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber(deviceAddress, -iRes);
        iRes = 0;
    }
    else
    {
        iRes = 1;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0)
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));

    channelBridge->logEnd("\t(%i, %i)", iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCCloseTable(int connectionId, int deviceAddress, int tableHandle, int *errorNumber)
{
    int res = 1;
    if (errorNumber == nullptr)
        return 0;

    if (DFCTableClose(connectionId, deviceAddress, tableHandle) == false)
    {
        ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
        // Wurde Channel-Objekt ermittelt.
        if (channelBridge == nullptr)
            return 0;

        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);
        res = 0;
    }

    return res;
}

DF_API_EXTERN int DF_API_CALL DFCTableSetFilter(int connectionId, int deviceAddress, int tableHandle, const char *value)
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, tableHandle);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false
        || value == nullptr || std::strlen(value) < 3) // mindestens Filter "a=b"
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        channelBridge->logEnd("\t%s\t(%i, %i)", (value == nullptr) ? "" : value, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        channelBridge->logEnd("\t%s\t(%i, %i)", value, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iRes = channelBridge->tableSetFilter(deviceAddress, tableHandle, value);
    if (iRes > IV_ERROR_SUCCESS)
    {
        int errorNumber = IV_ERROR_SUCCESS;
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                errorNumber = IV_ERROR_HANDLE;
                break;

            case 3:
                errorNumber = IV_ERROR_NULL_RESULT;
                break;

            case 4:
                errorNumber = IV_ERROR_NAME_UNKNOWN;
                break;

            case 28:
                errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                errorNumber = IV_ERROR_PROCESSING;
            }
        }

        channelBridge->setLastErrorNumber(deviceAddress, errorNumber);
        iRes = 0;
    }
    else if (iRes < IV_ERROR_SUCCESS)
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber(deviceAddress, -iRes);
        iRes = 0;
    }
    else
    {
        iRes = 1;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0)
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));

    channelBridge->logEnd("\t%s\t(%i, %i)", value, iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCSetFilter(int connectionId, int deviceAddress, int tableHandle, const char *value, int *errorNumber)
{
    int res = 1;
    if (errorNumber == nullptr)
        return 0;

    if (DFCTableSetFilter(connectionId, deviceAddress, tableHandle, value) == false)
    {
        ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
        // Wurde Channel-Objekt ermittelt.
        if (channelBridge == nullptr)
            return 0;

        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);
        res = 0;
    }

    return res;
}

DF_API_EXTERN int DF_API_CALL DFCTableGetFilter(int connectionId, int deviceAddress, int tableHandle, char *value, int *valueSize)
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, tableHandle);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false
        || value == nullptr || valueSize == nullptr || *valueSize < 2)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        channelBridge->logEnd("\t%s\t%i\t(%i, %i)", (value == nullptr) ? "" : value, (valueSize == nullptr) ? 0 : *valueSize, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        channelBridge->logEnd("\t%s\t%i\t(%i, %i)", value, *valueSize, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    std::string lValue;
    int iRes = channelBridge->tableGetFilter(deviceAddress, tableHandle, lValue);
    if (iRes > IV_ERROR_SUCCESS)
    {
        int errorNumber = IV_ERROR_SUCCESS;
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                errorNumber = IV_ERROR_HANDLE;
                break;

            case 28:
                errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                errorNumber = IV_ERROR_PROCESSING;
            }
        }

        iRes = 0;
    }
    else if (iRes > IV_ERROR_SUCCESS)
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber(deviceAddress, -iRes);
        iRes = 0;
    }
    else
    {
        *valueSize = static_cast<int>( lValue.copy(value, *valueSize - 1) );
        value[*valueSize] = 0; // Nullterminierung
        iRes = 1;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0)
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));

    channelBridge->logEnd("\t%s\t%i\t(%i, %i)", value, *valueSize, iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetFilter( int connectionId, int deviceAddress, int tableHandle, char *value, int *valueLength, int *errorNumber )
{
    int res = 1;
    if (errorNumber == nullptr)
        return 0;

    if (DFCTableGetFilter(connectionId, deviceAddress, tableHandle, value, valueLength) == false)
    {
        ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
        // Wurde Channel-Objekt ermittelt.
        if (channelBridge == nullptr)
            return 0;

        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);
        res = 0;
    }

    return res;
}

DF_API_EXTERN int DF_API_CALL DFCTableRemoveFilter(int connectionId, int deviceAddress, int tableHandle)
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, tableHandle);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        channelBridge->logEnd("\t(%i, %i)", 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        channelBridge->logEnd("\t(%i, %i)", 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iRes = channelBridge->tableRemoveFilter(deviceAddress, tableHandle);
    if (iRes > IV_ERROR_SUCCESS)
    {
        int errorNumber = IV_ERROR_SUCCESS;
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                errorNumber = IV_ERROR_HANDLE;
                break;

            case 28:
                errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                errorNumber = IV_ERROR_PROCESSING;
            }
        }

        iRes = 0;
    }
    else if (iRes < IV_ERROR_SUCCESS)
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber(deviceAddress, -iRes);
        iRes = 0;
    }
    else
    {
        iRes = 1;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0)
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));

    channelBridge->logEnd("\t(%i, %i)", iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCClearFilter( int connectionId, int deviceAddress, int tableHandle, int *errorNumber )
{
    int res = 1;
    if (errorNumber == nullptr)
        return 0;

    if (DFCTableRemoveFilter(connectionId, deviceAddress, tableHandle) == false)
    {
        ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
        // Wurde Channel-Objekt ermittelt.
        if (channelBridge == nullptr)
            return 0;

        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);
        res = 0;
    }

    return res;
}

DF_API_EXTERN int DF_API_CALL DFCTableGetRowCount(int connectionId, int deviceAddress, int tableHandle, int *currentCount, int *unsortedCount, int *deletedCount)
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, tableHandle);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false
        || currentCount == nullptr || unsortedCount == nullptr || deletedCount == nullptr)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        channelBridge->logEnd("\t%i\t%i\t%i\t(%i, %i)", (currentCount == nullptr) ? 0 : *currentCount, (unsortedCount == nullptr) ? 0 : *unsortedCount, (deletedCount == nullptr) ? 0 : *deletedCount, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        channelBridge->logEnd("\t%i\t%i\t%i\t(%i, %i)", *currentCount, *unsortedCount, *deletedCount, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iRes = channelBridge->tableGetRowCount(deviceAddress, tableHandle, reinterpret_cast<unsigned int&>(*currentCount), reinterpret_cast<unsigned int&>(*unsortedCount), reinterpret_cast<unsigned int&>(*deletedCount));
    if (iRes > IV_ERROR_SUCCESS)
    {
        int errorNumber = IV_ERROR_SUCCESS;
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                errorNumber = IV_ERROR_HANDLE;
                break;

            case 2:
                errorNumber = IV_ERROR_NAME_UNKNOWN;
                break;

            case 3:
                errorNumber = IV_ERROR_VALUE;
                break;

            case 28:
                errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                errorNumber = IV_ERROR_PROCESSING;
            }
        }

        channelBridge->setLastErrorNumber(deviceAddress, errorNumber);
        iRes = 0;
    }
    else if (iRes > IV_ERROR_SUCCESS)
    { 
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber(deviceAddress, -iRes);
        iRes = 0;
    }
    else
    {
        iRes = 1;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0)
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));

    channelBridge->logEnd("\t%i\t%i\t%i\t(%i, %i)", *currentCount, *unsortedCount, *deletedCount, iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCTableGetCurrentRow(int connectionId, int deviceAddress, int tableHandle, int *currentRow)
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, tableHandle);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false
        || currentRow == nullptr)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        channelBridge->logEnd("\t%i\t(%i, %i)", (currentRow == nullptr) ? 0 : *currentRow, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        channelBridge->logEnd("\t%i\t(%i, %i)", *currentRow, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iRes = channelBridge->tableGetCurrentRow(deviceAddress, tableHandle, reinterpret_cast<unsigned int&>(*currentRow));
    if (iRes > IV_ERROR_SUCCESS)
    {
        int errorNumber = IV_ERROR_SUCCESS;
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                errorNumber = IV_ERROR_HANDLE;
                break;

            case 2:
                errorNumber = IV_ERROR_NAME_UNKNOWN;
                break;

            case 3:
                errorNumber = IV_ERROR_VALUE;
                break;

            case 28:
                errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                errorNumber = IV_ERROR_PROCESSING;
            }
        }

        channelBridge->setLastErrorNumber(deviceAddress, errorNumber);
        iRes = 0;
    }
    else if (iRes < IV_ERROR_SUCCESS)
    { 
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber(deviceAddress, -iRes);
        iRes = 0;
    }
    else
    {
        iRes = 1;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0)
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));

    channelBridge->logEnd("\t%i\t(%i, %i)", *currentRow, iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCTableSetCurrentRow(int connectionId, int deviceAddress, int tableHandle, int offset, int direction)
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i\t%i\t%i", __func__, connectionId, deviceAddress, tableHandle, offset, direction);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false
        || (direction != 0 && direction != 1 && direction != 2))
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        channelBridge->logEnd("\t(%i, %i)", 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        channelBridge->logEnd("\t(%i, %i)", 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iRes = channelBridge->tableSetCurrentRow(deviceAddress, tableHandle, reinterpret_cast<unsigned int &>(offset), reinterpret_cast<unsigned short &>(direction));
    if (iRes > IV_ERROR_SUCCESS)
    {
        int errorNumber = IV_ERROR_SUCCESS;
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                errorNumber = IV_ERROR_HANDLE;
                break;

            case 3:
                errorNumber = IV_ERROR_BOF;
                break;

            case 4:
                errorNumber = IV_ERROR_EOF;
                break;

            case 28:
                errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                errorNumber = IV_ERROR_PROCESSING;
            }
        }

        channelBridge->setLastErrorNumber(deviceAddress, errorNumber);
        iRes = 0;
    }
    else if (iRes < IV_ERROR_SUCCESS)
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber(deviceAddress, -iRes);
        iRes = 0;
    }
    else
    {
        iRes = 1;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0)
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));

    channelBridge->logEnd("\t(%i, %i)", iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCSkip( int connectionId, int deviceAddress, int tableHandle, int offset, int origin, int *errorNumber )
{
    int res = 1;
    if (errorNumber == nullptr)
        return 0;

    if (DFCTableSetCurrentRow(connectionId, deviceAddress, tableHandle, offset, origin) == false)
    {
        ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
        // Wurde Channel-Objekt ermittelt.
        if (channelBridge == nullptr)
            return 0;

        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);
        res = 0;
    }

    return res;
}

DF_API_EXTERN int DF_API_CALL DFCTableSetCurrentRowData(int connectionId, int deviceAddress, int tableHandle, int flags, char separator, const char *data, int dataSize)
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i\t0x%08X\t%c", __func__, connectionId, deviceAddress, tableHandle, flags, separator);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false
        || flags < 0 || flags > 1
        || (separator != 0 && separator != '\t' && separator != ';') || data == nullptr || (separator == 0 && dataSize < 2))
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        if (separator == 0 && data != nullptr)
        {

        }
        channelBridge->logEnd("\t%s\t%i\t(%i, %i)", (data == nullptr) ? "" : data, dataSize, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        channelBridge->logEnd("\t%s\t%i\t(%i, %i)", data, dataSize, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iRes = channelBridge->tableSetCurrentRowData(deviceAddress, tableHandle, flags, separator, data, static_cast<unsigned short>(dataSize));
    if (iRes > IV_ERROR_SUCCESS)
    {
        int errorNumber = IV_ERROR_SUCCESS;
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                errorNumber = IV_ERROR_HANDLE;
                break;

            case 2:
                errorNumber = IV_ERROR_NAME_UNKNOWN;
                break;

            case 3:
                errorNumber = IV_ERROR_VALUE;
                break;

            case 28:
                errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                errorNumber = IV_ERROR_PROCESSING;
            }
        }

        channelBridge->setLastErrorNumber(deviceAddress, errorNumber);
        iRes = 0;
    }
    else if (iRes < IV_ERROR_SUCCESS)
    { 
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber(deviceAddress, -iRes);
        iRes = 0;
    }
    else
    {
        iRes = 1;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0)
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));

    channelBridge->logEnd("\t%s\t%i\t(%i, %i)", data, dataSize, iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCTableSetCurrentColumnData(int connectionId, int deviceAddress, int tableHandle, int flags, const char *name, const char *data, int dataSize)
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i\t0x%08X", __func__, connectionId, deviceAddress, tableHandle, flags);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false
        || flags < 0 || flags > 1
        || name == nullptr || data == nullptr 
        || (dataSize == 0 && strlen(data) == 0) 
        || (dataSize == 0 && strlen(data) > 250)
        || dataSize > 440) // Binaerfelder koennen als Hex-String geschickt werden.
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);
        channelBridge->logEnd("\t%s\t%s\t%i\t(%i, %i)", (name == nullptr) ? "" : name, (data == nullptr) ? "" : data, dataSize, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        channelBridge->logEnd("\t%s\t%s\t%i\t(%i, %i)", name, data, dataSize, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iRes = channelBridge->tableSetCurrentColumnData(deviceAddress, tableHandle, static_cast<unsigned short>(flags), name, 
        data, static_cast<unsigned short>(dataSize == 0 ? strlen(data) : dataSize));
    if (iRes > IV_ERROR_SUCCESS)
    {
        int errorNumber = IV_ERROR_SUCCESS;
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                errorNumber = IV_ERROR_HANDLE;
                break;

            case 2:
                errorNumber = IV_ERROR_NAME_UNKNOWN;
                break;

            case 3:
                errorNumber = IV_ERROR_VALUE;
                break;

            case 28:
                errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                errorNumber = IV_ERROR_PROCESSING;
            }
        }

        channelBridge->setLastErrorNumber(deviceAddress, errorNumber);
        iRes = 0;
    }
    else if (iRes < IV_ERROR_SUCCESS)
    { 
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber(deviceAddress, -iRes);
        iRes = 0;
    }
    else
    {
        iRes = 1;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0)
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));

    channelBridge->logEnd("\t%s\t%s\t%i\t(%i, %i)", name, data, dataSize, iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCSetField( int connectionId, int deviceAddress, int tableHandle, const char *name, const char *value, int *errorNumber )
{
    int res = 1;
    if (errorNumber == nullptr)
        return 0;

    int dataSize = static_cast<int>( strlen(value) );
    if (DFCTableSetCurrentColumnData(connectionId, deviceAddress, tableHandle, 0, name, value, dataSize) == false)
    {
        ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
        // Wurde Channel-Objekt ermittelt.
        if (channelBridge == nullptr)
            return 0;

        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);
        res = 0;
    }

    return res;
}

DF_API_EXTERN int DF_API_CALL DFCTableSetAllRowsToColumnData(int connectionId, int deviceAddress, int tableHandle, int flags, const char *name, const char *data, int dataSize)
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i\t0x%08X", __func__, connectionId, deviceAddress, tableHandle, flags);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false
        || flags < 0 || flags > 0
        || name == nullptr || data == nullptr 
        || (dataSize == 0 && strlen(data) == 0) 
        || (dataSize == 0 && strlen(data) > 250)
        || dataSize > 440) // Binaerfelder koennen als Hex-String geschickt werden.
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);
        channelBridge->logEnd("\t%s\t%s\t%i\t(%i, %i)", (name == nullptr) ? "" : name, (data == nullptr) ? "" : data, dataSize, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        channelBridge->logEnd("\t%s\t%s\t%i\t(%i, %i)", name, data, dataSize, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iRes = channelBridge->tableSetAllRowsToColumnData(deviceAddress, tableHandle, static_cast<unsigned short>(flags), name, 
        data, static_cast<unsigned short>(dataSize == 0 ? strlen(data) : dataSize));
    if (iRes > IV_ERROR_SUCCESS)
    {
        int errorNumber = IV_ERROR_SUCCESS;
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                errorNumber = IV_ERROR_HANDLE;
                break;

            case 2:
                errorNumber = IV_ERROR_NAME_UNKNOWN;
                break;

            case 3:
                errorNumber = IV_ERROR_VALUE;
                break;

            case 28:
                errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                errorNumber = IV_ERROR_PROCESSING;
            }
        }

        channelBridge->setLastErrorNumber(deviceAddress, errorNumber);
        iRes = 0;
    }
    else if (iRes < IV_ERROR_SUCCESS)
    { 
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber(deviceAddress, -iRes);
        iRes = 0;
    }
    else
    {
        iRes = 1;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0)
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));

    channelBridge->logEnd("\t%s\t%s\t%i\t(%i, %i)", name, data, dataSize, iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCTableGetCurrentRowData(int connectionId, int deviceAddress, int tableHandle, int flags, char separator, char *data, int *dataSize)
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i\t0x%08X\t%c", __func__, connectionId, deviceAddress, tableHandle, flags, separator);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false
        || flags < 0 || flags > 1
        || (separator != 0 && separator != '\t' && separator != ';') || data == nullptr || dataSize == nullptr || *dataSize < 2)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        if (separator == 0 && data != nullptr)
        {

        }
        channelBridge->logEnd("\t%s\t%i\t(%i, %i)", (data == nullptr) ? "" : data, (dataSize == nullptr) ? 0 : *dataSize, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        channelBridge->logEnd("\t%s\t%i\t(%i, %i)", data, *dataSize, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iRes = channelBridge->tableGetCurrentRowData(deviceAddress, tableHandle, flags, separator, data, reinterpret_cast<unsigned short&>(*dataSize));
    if (iRes > IV_ERROR_SUCCESS)
    {
        int errorNumber = IV_ERROR_SUCCESS;
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                errorNumber = IV_ERROR_HANDLE;
                break;

            case 2:
                errorNumber = IV_ERROR_NAME_UNKNOWN;
                break;

            case 3:
                errorNumber = IV_ERROR_VALUE;
                break;

            case 28:
                errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                errorNumber = IV_ERROR_PROCESSING;
            }
        }

        channelBridge->setLastErrorNumber(deviceAddress, errorNumber);
        iRes = 0;
    }
    else if (iRes < IV_ERROR_SUCCESS)
    { 
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber(deviceAddress, -iRes);
        iRes = 0;
    }
    else
    {
        iRes = 1;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0)
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));

    channelBridge->logEnd("\t%s\t%i\t(%i, %i)", data, *dataSize, iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCTableGetCurrentColumnData(int connectionId, int deviceAddress, int tableHandle, int flags, const char *name, char *data, int *dataSize)
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i\t0x%08X", __func__, connectionId, deviceAddress, tableHandle, flags);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false
        || flags < 0 || flags > 1
        || name == nullptr || data == nullptr || dataSize == nullptr || *dataSize < 2)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        channelBridge->logEnd("\t%s\t%s\t%i\t(%i, %i)", (name == nullptr) ? "" : name, (data == nullptr) ? "" : data, (dataSize == nullptr) ? 0 : *dataSize, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        channelBridge->logEnd("\t%s\t%s\t%i\t(%i, %i)", name, data, *dataSize, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iRes = channelBridge->tableGetCurrentColumnData(deviceAddress, tableHandle, flags, name, data, reinterpret_cast<unsigned short &>(*dataSize));
    if (iRes > IV_ERROR_SUCCESS)
    {
        int errorNumber = IV_ERROR_SUCCESS;
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                errorNumber = IV_ERROR_HANDLE;
                break;

            case 2:
                errorNumber = IV_ERROR_NAME_UNKNOWN;
                break;

            case 28:
                errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                errorNumber = IV_ERROR_PROCESSING;
            }
        }

        channelBridge->setLastErrorNumber(deviceAddress, errorNumber);
        iRes = 0;
    }
    else if (iRes < IV_ERROR_SUCCESS)
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber(deviceAddress, -iRes);
        iRes = 0;
    }
    else
    {
        iRes = 1;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0)
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));

    channelBridge->logEnd("\t%s\t%s\t%i\t(%i, %i)", name, data, *dataSize, iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCGetField( int connectionId, int deviceAddress, int tableHandle, const char *name, char *value, int *valueLength, int *errorNumber )
{
    int res = 1;
    if (errorNumber == nullptr)
        return 0;

    if (DFCTableGetCurrentColumnData(connectionId, deviceAddress, tableHandle, 0, name, value, valueLength) == false)
    {
        ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
        // Wurde Channel-Objekt ermittelt.
        if (channelBridge == nullptr)
            return 0;

        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);
        res = 0;
    }

    return res;
}

DF_API_EXTERN int DF_API_CALL DFCTableAppendRowData(int connectionId, int deviceAddress, int tableHandle, int flags, char separator, const char *data, int dataSize)
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i\t0x%08X\t%c", __func__, connectionId, deviceAddress, tableHandle, flags, separator);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false
        || flags < 0 || flags > 0
        || (separator != 0 && separator != '\t' && separator != ';') || data == nullptr || (separator == 0 && dataSize < 2))
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        if (separator == 0 && data != nullptr)
        {

        }
        channelBridge->logEnd("\t%s\t%i\t(%i, %i)", (data == nullptr) ? "" : data, dataSize, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        channelBridge->logEnd("\t%s\t%i\t(%i, %i)", data, dataSize, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iRes = channelBridge->tableAppendRowData(deviceAddress, tableHandle, flags, separator, data, static_cast<unsigned short>(dataSize));
    if (iRes > IV_ERROR_SUCCESS)
    {
        int errorNumber = IV_ERROR_SUCCESS;
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                errorNumber = IV_ERROR_HANDLE;
                break;

            case 2:
                errorNumber = IV_ERROR_NAME_UNKNOWN;
                break;

            case 3:
                errorNumber = IV_ERROR_VALUE;
                break;

            case 28:
                errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                errorNumber = IV_ERROR_PROCESSING;
            }
        }

        channelBridge->setLastErrorNumber(deviceAddress, errorNumber);
        iRes = 0;
    }
    else if (iRes < IV_ERROR_SUCCESS)
    { 
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber(deviceAddress, -iRes);
        iRes = 0;
    }
    else
    {
        iRes = 1;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0)
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));

    channelBridge->logEnd("\t%s\t%i\t(%i, %i)", data, dataSize, iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCTableDeleteCurrentRow(int connectionId, int deviceAddress, int tableHandle, int flags)
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i\t0x%08X", __func__, connectionId, deviceAddress, tableHandle, flags);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false
        || flags < 0 || flags > 1)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        channelBridge->logEnd("\t(%i, %i)", 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        channelBridge->logEnd("\t(%i, %i)", 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iRes = channelBridge->tableDeleteCurrentRow(deviceAddress, tableHandle, flags);
    if (iRes > IV_ERROR_SUCCESS)
    {
        int errorNumber = IV_ERROR_SUCCESS;
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                errorNumber = IV_ERROR_HANDLE;
                break;

            case 2:
                errorNumber = IV_ERROR_NAME_UNKNOWN;
                break;

            case 3:
                errorNumber = IV_ERROR_VALUE;
                break;

            case 28:
                errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                errorNumber = IV_ERROR_PROCESSING;
            }
        }

        channelBridge->setLastErrorNumber(deviceAddress, errorNumber);
        iRes = 0;
    }
    else if (iRes < IV_ERROR_SUCCESS)
    { 
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber(deviceAddress, -iRes);
        iRes = 0;
    }
    else
    {
        iRes = 1;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0)
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));

    channelBridge->logEnd("\t(%i, %i)", iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCTableDeleteAvailableRows(int connectionId, int deviceAddress, int tableHandle, int flags)
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i\t0x%08X", __func__, connectionId, deviceAddress, tableHandle, flags);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false
        || flags < 0 || flags > 1)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        channelBridge->logEnd("\t(%i, %i)", 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        channelBridge->logEnd("\t(%i, %i)", 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iRes = channelBridge->tableDeleteAvailableRows(deviceAddress, tableHandle, flags);
    if (iRes > IV_ERROR_SUCCESS)
    {
        int errorNumber = IV_ERROR_SUCCESS;
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                errorNumber = IV_ERROR_HANDLE;
                break;

            case 2:
                errorNumber = IV_ERROR_NAME_UNKNOWN;
                break;

            case 3:
                errorNumber = IV_ERROR_VALUE;
                break;

            case 28:
                errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                errorNumber = IV_ERROR_PROCESSING;
            }
        }

        channelBridge->setLastErrorNumber(deviceAddress, errorNumber);
        iRes = 0;
    }
    else if (iRes < IV_ERROR_SUCCESS)
    { 
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        channelBridge->setLastErrorNumber(deviceAddress, -iRes);
        iRes = 0;
    }
    else
    {
        iRes = 1;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0)
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));

    channelBridge->logEnd("\t(%i, %i)", iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}
