#include "api/df_setup_records.h"
#include "datafox_library_iv.h"

DF_API_EXTERN_DECORATE int DF_API_CALL DFCLoadDatensatzbeschreibung( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
    {
        if (errorNumber)
            *errorNumber = IV_ERROR_CHANNEL;

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false || errorNumber == nullptr)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        if (errorNumber)
            *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);

        channelBridge->logEnd("\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);
        
        channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iRes = channelBridge->getTableDescription(deviceAddress, FALSE, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if (iRes == 0)
    {
        channelBridge->setLastErrorNumber(deviceAddress, *errorNumber);
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));
    }

    channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCDatBCnt( int connectionId )
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i", __func__, connectionId);

    int iRes = channelBridge->GetTableCount(FALSE);

    channelBridge->logEnd("\t(%i, %i)", iRes, channelBridge->getLastErrorNumber(0));

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCDatBDatensatz( int connectionId, int number, char *name, int *fieldCount )
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i", __func__, connectionId, number);

    // zulaessige Bereiche pruefen
    if (((number >= 0 && number < MAX_DATENSATZ_BESCHREIBUNG) == false && (number >= 100 && number <= 103) == false && (number >= 200 && number <= 207) == false)
        || name == nullptr || fieldCount == nullptr)
    {
        channelBridge->setLastErrorNumber(0, IV_ERROR_FUNCTION_PARAMETER);

        channelBridge->logEnd("\t\t\t(%i, %i)", 0, channelBridge->getLastErrorNumber(0));
        return 0;
    }

    std::string szTableName;
    unsigned char ucFieldCount;

    int iRes = channelBridge->GetTableRecord(number, szTableName, &ucFieldCount);

    if (iRes)
    {
        strcpy(name, szTableName.c_str());
        *fieldCount = ucFieldCount;
    }

    channelBridge->logEnd("\t%s\t%i\t(%i, %i)", name, *fieldCount, iRes, channelBridge->getLastErrorNumber(0));
    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCDatBFeld( int connectionId, int dataBNumber, int number, char *name, int *type, int *size )
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, dataBNumber, number);

    // zulaessige Bereiche pruefen
    if (((number >= 0 && number < MAX_DATENSATZ_BESCHREIBUNG) == false && (number >= 100 && number <= 103) == false && (number >= 200 && number <= 207) == false)
        || (number >= 0 && number < MAX_FELD) == false
        || name == nullptr || type == nullptr || size == nullptr)
    {
        channelBridge->setLastErrorNumber(0, IV_ERROR_FUNCTION_PARAMETER);

        channelBridge->logEnd("\t\t\t\t(%i, %i)", 0, channelBridge->getLastErrorNumber(0));
        return 0;
    }

    std::string szFieldName;
    unsigned char ucType, ucLength;

    int iRes = channelBridge->GetTableField(dataBNumber, number, szFieldName,
        &ucType, &ucLength, FALSE);

    if (iRes)
    {
        strcpy(name, szFieldName.c_str());
        *type = ucType;
        *size = ucLength;
    }

    channelBridge->logEnd("\t%s\t%i\t%i\t(%i, %i)", name, *type, *size, iRes, channelBridge->getLastErrorNumber(0));

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCComCollectData( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
    {
        if (errorNumber)
            *errorNumber = IV_ERROR_CHANNEL;

        return -1;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false || errorNumber == nullptr)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        if (errorNumber)
            *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);

        channelBridge->logEnd("\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber(deviceAddress));
        return -1;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);
        
        channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, -1, channelBridge->getLastErrorNumber(deviceAddress));
        return -1;
    }

    int iRes = channelBridge->collectData(deviceAddress, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if (iRes == 0)
    {
        channelBridge->setLastErrorNumber(deviceAddress, *errorNumber);
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));
    }

    channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber(deviceAddress));
    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCComGetDatensatz( int connectionId, unsigned char *buffer, int *bufferLength, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
    {
        if (errorNumber)
            *errorNumber = IV_ERROR_CHANNEL;

        return -1;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i", __func__, connectionId);

    // zulaessige Bereiche pruefen
    if (buffer == nullptr || bufferLength == nullptr || errorNumber == nullptr)
    {
        channelBridge->setLastErrorNumber(0, IV_ERROR_FUNCTION_PARAMETER);

        if (errorNumber)
            *errorNumber = channelBridge->getLastErrorNumber(0);

        channelBridge->logEnd("\t\t\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber(0));
        return -1;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(0, IV_ERROR_CHANNEL_CLOSED);
        *errorNumber = channelBridge->getLastErrorNumber(0);
        
        channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, -1, channelBridge->getLastErrorNumber(0));
        return -1;
    }

    int iRes = channelBridge->getDatensatz(buffer, (std::size_t &)*bufferLength, (int32_t &)*errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if (iRes == 0)
    {
        channelBridge->setLastErrorNumber(0, *errorNumber);
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(0));
    }

    switch ( iRes )
    {
    case 1:
    case 2: {

        // Daten in Puffer uebergeben
        unsigned char ucTmp[MAX_MB_PAKET_DATA * 3 + 1];
        int i;

        for (i=0; i < *bufferLength; i++)
        {
            sprintf((char*)&ucTmp[i * 3], "%02X ", (unsigned char)buffer[i]);
        }
        ucTmp[i * 3] = 0;

        channelBridge->logAppend("\t%s\t%i", ucTmp, *bufferLength);
        break;}

    default:
        channelBridge->logAppend("\t%s\t%i", "", 0);
    }

    channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber(0));
    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCReadRecord( int connectionId, int deviceAddress, unsigned char *buffer, int *bufferLength, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
    {
        if (errorNumber)
            *errorNumber = IV_ERROR_CHANNEL;

        return -1;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false 
        || buffer == nullptr || bufferLength == nullptr || errorNumber == nullptr)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        if (errorNumber)
            *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);

        channelBridge->logEnd("\t\t\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber(deviceAddress));
        return -1;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);
        
        channelBridge->logEnd("\t\t\t%i\t(%i, %i)", *errorNumber, -1, channelBridge->getLastErrorNumber(deviceAddress));
        return -1;
    }

    int res = channelBridge->readRecord(deviceAddress, buffer, (std::size_t &)*bufferLength, (int32_t &)*errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if (res == -1)
    {
        channelBridge->setLastErrorNumber(deviceAddress, *errorNumber);
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));

        channelBridge->logAppend("\t\t");
    }
    else if (res == 0)
    {
        channelBridge->logAppend("\t\t");
    }
    else
    {
        // Daten fuer Log in Puffer uebergeben
        unsigned char ucTmp[MAX_MB_PAKET_DATA * 3 + 1];
        int i;
        for (i=0; i < *bufferLength; i++)
        {
            sprintf((char*)&ucTmp[i * 3], "%02X ", (unsigned char)buffer[i]);
        }
        ucTmp[i * 3] = 0;

        channelBridge->logAppend("\t%s\t%i", ucTmp, *bufferLength);
    }

    channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, res, channelBridge->getLastErrorNumber(deviceAddress));

    return res;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCQuitRecord( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
    {
        if (errorNumber)
            *errorNumber = IV_ERROR_CHANNEL;

        return -1;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false || errorNumber == nullptr)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        if (errorNumber)
            *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);

        channelBridge->logEnd("\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber(deviceAddress));
        return -1;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);
        
        channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, -1, channelBridge->getLastErrorNumber(deviceAddress));
        return -1;
    }

    int res = channelBridge->quitRecord(deviceAddress, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if (res == -1)
    {
        channelBridge->setLastErrorNumber(deviceAddress, *errorNumber);
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));
    }

    channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, res, channelBridge->getLastErrorNumber(deviceAddress));

    return res;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCComClearData( int connectionId, int deviceAddress )
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
        return 0;

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        channelBridge->logEnd("\t(%i, %i)", 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        
        channelBridge->logEnd("\t(%i, %i)", 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iError = IV_ERROR_SUCCESS;
    int iRes = channelBridge->clearData(deviceAddress, &iError);
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if (iRes == 0)
    {
        channelBridge->setLastErrorNumber(deviceAddress, iError);
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));
    }

    channelBridge->logEnd("\t(%i, %i)", iRes, channelBridge->getLastErrorNumber(deviceAddress));
    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCRestoreRecords( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
    {
        if (errorNumber)
            *errorNumber = IV_ERROR_CHANNEL;

        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false || errorNumber == nullptr)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        if (errorNumber)
            *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);

        channelBridge->logEnd("\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);
        
        channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, 0, channelBridge->getLastErrorNumber(deviceAddress));
        return 0;
    }

    int iRes = channelBridge->restoreRecords(deviceAddress, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if (iRes == 0)
    {
        channelBridge->setLastErrorNumber(deviceAddress, *errorNumber);
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));
    }

    channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber(deviceAddress));
    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCCheckData( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (channelBridge == nullptr)
    {
        if (errorNumber)
            *errorNumber = IV_ERROR_CHANNEL;

        return -1;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber(channelBridge->isValidDeviceAddress(deviceAddress) ? deviceAddress : 0, IV_ERROR_SUCCESS);

    channelBridge->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (channelBridge->isValidDeviceAddress(deviceAddress) == false || errorNumber == nullptr)
    {
        if (channelBridge->isValidDeviceAddress(deviceAddress) == false)
            deviceAddress = 0;

        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_FUNCTION_PARAMETER);

        if (errorNumber)
            *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);

        channelBridge->logEnd("\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber(deviceAddress));
        return -1;
    }

    // Fehlerobjekt initalisieren.
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (channelBridge->isOpen() == false)
    {
        channelBridge->setLastErrorNumber(deviceAddress, IV_ERROR_CHANNEL_CLOSED);
        *errorNumber = channelBridge->getLastErrorNumber(deviceAddress);
        
        channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, -1, channelBridge->getLastErrorNumber(deviceAddress));
        return -1;
    }

    int iRes = channelBridge->checkData(deviceAddress, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if (iRes == -1)
    {
        channelBridge->setLastErrorNumber(deviceAddress, *errorNumber);
        channelBridge->resetWithError(channelBridge->getLastErrorNumber(deviceAddress));
    }

    channelBridge->logEnd("\t%i\t(%i, %i)", *errorNumber, iRes, channelBridge->getLastErrorNumber(deviceAddress));

    return iRes;
}
