/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/

#include "datafox/communication/module_hardware.h"
#include <stdio.h>
#include <cstring>

// ##################################################################
/**
* @brief Initialisieren der Membervariablen.
*
* @param id Nummer der Kanalklasse fuer z.B. Trace.
*/
Datafox::Communication::Module_hardware::Module_hardware( int connection_id, uint8_t device_address )
    : Module_base( connection_id, device_address ) 
{

}

// ##################################################################
/**
* @brief Destruktor.
*
*/
Datafox::Communication::Module_hardware::~Module_hardware() 
{
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*/
void Datafox::Communication::Module_hardware::error_module( uint16_t module, uint16_t error, std::string &text )
{
    trace( "%s(%i):INFO(3): %s( %u, %u, %p )\n", __FILE__, __LINE__, __func__, module, error, text.c_str() );

    // Ist IV_MODULE_HARDWARE ?
    if ( module == IV_MODULE_HARDWARE )
    {

    }

    // Ermittlung des Fehlertextes an Basisklasse abgeben.
    Module_base::error_module( module, error, text );
}


// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*/
void Datafox::Communication::Module_hardware::error_function( uint16_t module, uint16_t func, uint16_t error, std::string &text )
{
    trace( "%s(%i):INFO(3): %s( %u, %u, %u, %p )\n", __FILE__, __LINE__, __func__, module, func, error, text.c_str() );

    if ( module == IV_MODULE_HARDWARE )
    {
        // Sonderfehler für Passwortschutz abhandeln.
        if ( error == 28 )
        {
            // Die Modulfunktionen liefern den Fehlercode 28, wenn kein oder ein falsches Passwort verwendet wurde.
            // Es ist in diesem Fall zuerst das richtige Passwort für die Kommunikation erforderlich.
            text = boost::str( boost::format( "( %u ): Korrektes Passwort für die Kommunikation erforderlich.") % error );
            return;
        }
        else if ( error == 29 )
        {
            // Die Modulfunktionen liefern den Fehlercode 29, wenn 3 mal ein falsches Passwort verwendet wurde.
            // Die Befehlsausführung ist danach für 60 Sekunden blockiert.
            text = boost::str( boost::format( "( %u ): Gerät ist aufgrund von fehlerhaften Passworteingaben für 60 Sekunden gesperrt.") % error );
            return;
        }

        switch ( func )
        {
        case IV_FUNCTION_PING:
            // Setzen/Ermitteln des Geraetetyp
        case IV_FUNCTION_SET_TYPE_DEVICE:
        case IV_FUNCTION_GET_TYPE_DEVICE:

            // Setzen/Ermitteln des Boardtyp (Redesignversion z.B.: 0x13) (in Nibbles)
        case IV_FUNCTION_SET_TYPE_BOARD:
        case IV_FUNCTION_GET_TYPE_BOARD:

            // Setzen/Ermitteln der Boardkomponente: RS485 fuer Kommunikation mit PC
        case IV_FUNCTION_SET_COMPONENT_RS485_0:
        case IV_FUNCTION_GET_COMPONENT_RS485_0:

            // Setzen/Ermitteln der Boardkomponente: TCP/IP
        case IV_FUNCTION_SET_TYPE_TCPIP:
        case IV_FUNCTION_GET_TYPE_TCPIP:

            // Setzen/Ermitteln der Boardkomponente: Modem MC35i/MC55
        case IV_FUNCTION_SET_TYPE_MODEM_MC:
        case IV_FUNCTION_GET_TYPE_MODEM_MC:

            // Setzen/Ermitteln der Boardkomponente: RS485 fuer Zutrittskontrolle
        case IV_FUNCTION_SET_COMPONENT_RS485_1:
        case IV_FUNCTION_GET_COMPONENT_RS485_1:

            // Setzen/Ermitteln des Transpondertyps
        case IV_FUNCTION_SET_TYPE_TRANSPONDER:
        case IV_FUNCTION_GET_TYPE_TRANSPONDER:

            // Setzen/Ermitteln des Displaytyp
        case IV_FUNCTION_SET_TYPE_DISPLAY:
        case IV_FUNCTION_GET_TYPE_DISPLAY:

            // Setzen/Ermitteln des Tastaturtyps
        case IV_FUNCTION_SET_TYPE_KEYPAD:
        case IV_FUNCTION_GET_TYPE_KEYPAD:

            // Setzen/Ermitteln der Multimediakarte
        case IV_FUNCTION_SET_TYPE_MMC:
        case IV_FUNCTION_GET_TYPE_MMC:

            // Setzen/Ermitteln des GPS - Typs
        case IV_FUNCTION_SET_TYPE_GPS:
        case IV_FUNCTION_GET_TYPE_GPS:

            // Setzen/Ermitteln der USB - Komponente
        case IV_FUNCTION_SET_COMPONENT_USB:
        case IV_FUNCTION_GET_COMPONENT_USB:

            // Setzen/Ermitteln der Hardwareconfigurationsnummer
        case IV_FUNCTION_SET_HW_CONFIG:
        case IV_FUNCTION_GET_HW_CONFIG:

            // Setzen/Ermitteln des Funkmodultyps
        case IV_FUNCTION_SET_TYPE_RADIO:
        case IV_FUNCTION_GET_TYPE_RADIO:

            // Setzen/Ermitteln des Funkuhrmodultyps
        case IV_FUNCTION_SET_TYPE_DCF77:
        case IV_FUNCTION_GET_TYPE_DCF77:

            // Setzen/Ermitteln der analogen Eingaenge - Komponente
        case IV_FUNCTION_SET_COMPONENT_IN_ANALOG:
        case IV_FUNCTION_GET_COMPONENT_IN_ANALOG:

            // Setzen/Ermitteln der digitalen Eingaenge - Komponente
        case IV_FUNCTION_SET_COMPONENT_IN_DIGITAL:
        case IV_FUNCTION_GET_COMPONENT_IN_DIGITAL:

            // Setzen/Ermitteln der digitalen Ausgaenge - Komponente
        case IV_FUNCTION_SET_COMPONENT_OUT_DIGITAL:
        case IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL:

            // Setzen/Ermitteln des Fingerprint + Typs
        case IV_FUNCTION_SET_TYPE_FINGERPRINT:
        case IV_FUNCTION_GET_TYPE_FINGERPRINT:

        case IV_FUNCTION_SET_TYPE_EXTERNAL_READER:
        case IV_FUNCTION_GET_TYPE_EXTERNAL_READER:

            // Optionen schreiben/lesen
        case IV_FUNCTION_SET_OPTION_IOMODULE:
        case IV_FUNCTION_GET_OPTION_IOMODULE:

        case IV_FUNCTION_SET_OPTION_COMSERVER:
        case IV_FUNCTION_GET_OPTION_COMSERVER:

        case IV_FUNCTION_SET_OPTION_TBADAPTION:
        case IV_FUNCTION_GET_OPTION_TBADAPTION:
            text = boost::str( boost::format( "( %u ): Nicht eingetragener Funktionsfehler!") % error );
            break;

        default:
            text = boost::str( boost::format( "( %u ): Nicht eingetragener Fehlerblock für Funktion!") % error );
            break;
        }

        return;
    }

    // Ermittlung des Fehlertextes an Basisklasse abgeben.
    Module_base::error_function( module, func, error, text );
}

//#########################################################
//###################### HARDWARE #########################
//#########################################################

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_hardware::remote_call( uint16_t module, uint16_t func, uint8_t *data, uint16_t &count )
{
    if ( module == IV_MODULE_HARDWARE )
    {
        switch ( func )
        {
        case IV_FUNCTION_PING:
            return ping( get_device_address_sender(), data, &count );

            // Setzen/Ermitteln des Geraetetyp
        case IV_FUNCTION_SET_TYPE_DEVICE:
            return write_device_type( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_DEVICE:
            return read_device_type( get_device_address_sender(), (unsigned char&)data[0] );

            // Setzen/Ermitteln des Boardtyp (Redesignversion z.B.: 0x13) (in Nibbles)
        case IV_FUNCTION_SET_TYPE_BOARD:
            return write_board_type( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_BOARD:
            return read_board_type( get_device_address_sender(), (unsigned char&)data[0] );

            // Setzen/Ermitteln der Boardkomponente: RS485 fuer Kommunikation mit PC
        case IV_FUNCTION_SET_COMPONENT_RS485_0:
            return write_rs485_communication_component( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_COMPONENT_RS485_0:
            return read_rs485_communication_component( get_device_address_sender(), (unsigned char&)data[0] );

            // Setzen/Ermitteln der Boardkomponente: TCP/IP
        case IV_FUNCTION_SET_TYPE_TCPIP:
            return write_network_adapter_type( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_TCPIP:
            return read_network_adapter_type( get_device_address_sender(), (unsigned char&)data[0] );

            // Setzen/Ermitteln der Boardkomponente: Modem MC35i/MC55
        case IV_FUNCTION_SET_TYPE_MODEM_MC:
            return write_modem_type( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_MODEM_MC:
            return read_modem_type( get_device_address_sender(), (unsigned char&)data[0] );

            // Setzen/Ermitteln der Boardkomponente: RS485 fuer Zutrittskontrolle
        case IV_FUNCTION_SET_COMPONENT_RS485_1:
            return write_rs485_access_control_component( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_COMPONENT_RS485_1:
            return read_rs485_access_control_component( get_device_address_sender(), (unsigned char&)data[0] );

            // Setzen/Ermitteln des Transpondertyps
        case IV_FUNCTION_SET_TYPE_TRANSPONDER:
            return write_transponder_reader_type( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_TRANSPONDER:
            return read_transponder_reader_type( get_device_address_sender(), (unsigned char&)data[0] );

            // Setzen/Ermitteln des Displaytyp
        case IV_FUNCTION_SET_TYPE_DISPLAY:
            return write_display_type( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_DISPLAY:
            return read_display_type( get_device_address_sender(), (unsigned char&)data[0] );

            // Setzen/Ermitteln des Tastaturtyps
        case IV_FUNCTION_SET_TYPE_KEYPAD:
            return write_keypad_type( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_KEYPAD:
            return read_keypad_type( get_device_address_sender(), (unsigned char&)data[0] );

            // Setzen/Ermitteln der Multimediakarte
        case IV_FUNCTION_SET_TYPE_MMC:
            return write_mmc_type( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_MMC:
            return read_mmc_type( get_device_address_sender(), (unsigned char&)data[0] );

            // Setzen/Ermitteln des GPS - Typs
        case IV_FUNCTION_SET_TYPE_GPS:
            return write_gps_type( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_GPS:
            return read_gps_type( get_device_address_sender(), (unsigned char&)data[0] );

            // Setzen/Ermitteln der USB - Komponente
        case IV_FUNCTION_SET_COMPONENT_USB:
            return write_usb_component( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_COMPONENT_USB:
            return read_usb_component( get_device_address_sender(), (unsigned char&)data[0] );

            // Setzen/Ermitteln der Hardwareconfigurationsnummer
        case IV_FUNCTION_SET_HW_CONFIG:
            return write_hardware_info_version( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_HW_CONFIG:
            return read_hardware_info_version( get_device_address_sender(), (unsigned char&)data[0] );

            // Setzen/Ermitteln des Funkmodultyps
        case IV_FUNCTION_SET_TYPE_RADIO:
            return write_radio_type( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_RADIO:
            return read_radio_type( get_device_address_sender(), (unsigned char&)data[0] );

            // Setzen/Ermitteln des Funkuhrmodultyps
        case IV_FUNCTION_SET_TYPE_DCF77:
            return write_DCF77_type( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_DCF77:
            return read_DCF77_type( get_device_address_sender(), (unsigned char&)data[0] );

            // Setzen/Ermitteln der analogen Eingaenge - Komponente
        case IV_FUNCTION_SET_COMPONENT_IN_ANALOG:
            return write_analog_input_component( get_device_address_sender(), *(unsigned int*)data, *(unsigned short*)&data[4] );

        case IV_FUNCTION_GET_COMPONENT_IN_ANALOG:
            return read_analog_input_component( get_device_address_sender(), (unsigned int&)data[0] );

            // Setzen/Ermitteln der digitalen Eingaenge - Komponente
        case IV_FUNCTION_SET_COMPONENT_IN_DIGITAL:
            return write_digital_input_component( get_device_address_sender(), *(unsigned int*)data, *(unsigned short*)&data[4] );

        case IV_FUNCTION_GET_COMPONENT_IN_DIGITAL:
            return read_digital_input_component( get_device_address_sender(), (unsigned int&)data[0] );

            // Setzen/Ermitteln der digitalen Ausgaenge - Komponente
        case IV_FUNCTION_SET_COMPONENT_OUT_DIGITAL:
            return write_digital_output_component( get_device_address_sender(), *(unsigned int*)data, *(unsigned short*)&data[4] );

        case IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL:
            return read_digital_output_component( get_device_address_sender(), (unsigned int&)data[0] );

            // Setzen/Ermitteln des Fingerprint + Typs
        case IV_FUNCTION_SET_TYPE_FINGERPRINT:
            return write_fingerprint_scanner_type( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_FINGERPRINT:
            return read_fingerprint_scanner_type( get_device_address_sender(), (unsigned char&)data[0] );

        case IV_FUNCTION_SET_TYPE_EXTERNAL_READER:
            return write_external_transponder_reader_type( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_TYPE_EXTERNAL_READER:
            return read_external_transponder_reader_type( get_device_address_sender(), (unsigned char&)data[0] );

            // Optionen schreiben/lesen
        case IV_FUNCTION_SET_OPTION_IOMODULE:
            return write_io_module_option( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_OPTION_IOMODULE:
            return read_io_module_option( get_device_address_sender(), (unsigned char&)data[0] );

        case IV_FUNCTION_SET_OPTION_COMSERVER:
            return write_com_server_option( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_OPTION_COMSERVER:
            return read_com_server_option( get_device_address_sender(), (unsigned char&)data[0] );

        case IV_FUNCTION_SET_OPTION_TBADAPTION:
            return write_timeboy_adaption_option( get_device_address_sender(), data[0], *(unsigned short*)&data[1] );

        case IV_FUNCTION_GET_OPTION_TBADAPTION:
            return read_timeboy_adaption_option( get_device_address_sender(), (unsigned char&)data[0] );

        case IV_FUNCTION_INSPECTION_STATION:
        {
            unsigned char address;
            std::string answer( IV_PACKET_LENGTH_DATA, 0 );
            int size_answer;
            int to_pending = 0;
            int res = inspection_station_command( get_device_address_sender(),
                                             data[0], (char *)&data[1],
                                            std::strlen( (const char *)&data[1] ),
                                             address, &answer[0], size_answer, to_pending );
            if ( res == IV_ERROR_SUCCESS )
            {
                data[0] = address;
                answer.resize( size_answer );
                answer.copy( (char *)&data[1], answer.size() );
                *(int *)&data[1 + answer.size()] = size_answer;
            }

            return res;
        }

        case IV_FUNCTION_INSPECTION_STATION_OK:
            return reply_inspection_station_command( get_device_address_sender() );

        case IV_FUNCTION_SET_OPTION_TPWRITE:
            return write_transponder_write_option( get_device_address_sender(), data[0], *(unsigned short *)&data[1] );

        case IV_FUNCTION_GET_OPTION_TPWRITE:
            return read_transponder_write_option( get_device_address_sender(), (unsigned char &)data[0] );

        case IV_FUNCTION_SET_OPTION_HTTP:
            return write_http_option( get_device_address_sender(), data[0], *(unsigned short *)&data[1] );

        case IV_FUNCTION_GET_OPTION_HTTP:
            return read_http_option( get_device_address_sender(), (unsigned char &)data[0] );

        case IV_FUNCTION_SET_OPTION_LANGUAGE:
            return write_localization_option( get_device_address_sender(), data[0], *(unsigned short *)&data[1] );

        case IV_FUNCTION_GET_OPTION_LANGUAGE:
            return read_localization_option( get_device_address_sender(), (unsigned char &)data[0] );

        case IV_FUNCTION_SET_OPTION_BIOKEY_UPDATE:
            return write_fingerprint_scanner_update_option( get_device_address_sender(), data[0], *(unsigned short *)&data[1] );

        case IV_FUNCTION_GET_OPTION_BIOKEY_UPDATE:
            return read_fingerprint_scanner_update_option( get_device_address_sender(), (unsigned char &)data[0] );

        case IV_FUNCTION_SET_SETUP_OPTIONS:
            return write_setup_options( get_device_address_sender(), *(unsigned short *)&data[0], *(unsigned short *)&data[2] );

        case IV_FUNCTION_GET_SETUP_OPTIONS:
            return read_setup_options( get_device_address_sender(), (unsigned short &)data[0], (unsigned short &)data[2] );

        case IV_FUNCTION_SET_TYPE_TOUCH:
            return write_touchscreen_type( get_device_address_sender(), data[0], *(unsigned short *)&data[1] );

        case IV_FUNCTION_GET_TYPE_TOUCH:
            return read_touchscreen_type( get_device_address_sender(), (unsigned char &)data[0] );

        case IV_FUNCTION_SET_TYPE_SENSOR:
            return write_sensor_type( get_device_address_sender(), data[0], *(unsigned short *)&data[1] );

        case IV_FUNCTION_GET_TYPE_SENSOR:
            return read_sensor_type( get_device_address_sender(), (unsigned char &)data[0] );

        case IV_FUNCTION_SET_COMPONENT_BATTERY:
            return write_battery_component( get_device_address_sender(), data[0], *(unsigned short *)&data[1] );

        case IV_FUNCTION_GET_COMPONENT_BATTERY:
            return read_battery_component( get_device_address_sender(), (unsigned char &)data[0] );

        case IV_FUNCTION_SET_COMPONENT_CHARGE:
            return write_battery_charger_component( get_device_address_sender(), data[0], *(unsigned short *)&data[1] );

        case IV_FUNCTION_GET_COMPONENT_CHARGE:
            return read_battery_charger_component( get_device_address_sender(), (unsigned char &)data[0] );

        case IV_FUNCTION_GET_HARDWARE_INFO:
            return read_hardware_info( get_device_address_sender(), (StructHardwareInfo &)data[0] );

        case IV_FUNCTION_SET_PAYABLE_OPTIONS:
            return write_buy_options( get_device_address_sender(), data[0], data[1], *(unsigned short *)&data[2] );

        case IV_FUNCTION_GET_PAYABLE_OPTIONS:
            return read_buy_options( get_device_address_sender(), data[0], data[2] );

        default: // Funktion nicht definiert
            return 0x0200;
        }
    }

    return Module_base::remote_call( module, func, data, count );
}

// ##################################################################
/**
* @brief Ermitteln des Geraetetyps.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Geraetetyp
*             0 = PZE - Master IV
*             1 = Zutritts - Master IV
*             2 = BDE - Master IV
*           ... = ...
*/
int Datafox::Communication::Module_hardware::read_device_type( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %u, %u )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DEVICE, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DEVICE, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof( uint8_t ) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__ );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char type = get_property( "hardware_info.type_device", IV_VALUE_TYPE_DEVICE_UNKNOWN );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DEVICE, &type, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt die vorliegende Board-Revisionsnummer.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Revisionsnummer in der Form (HiNibble Majornummer).(LoNibble Minornummer)
*             z. B. 0x14 -> Major 1, Minor (Redesign Nr.) 4
*/
int Datafox::Communication::Module_hardware::read_board_type(unsigned char device_address, unsigned char& val)
{
    trace( "%s(%i):INFO(3): %s( %u, %u )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_BOARD, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_BOARD, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char type = get_property( "hardware_info.type_board", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_BOARD, &type, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int Datafox::Communication::Module_hardware::read_rs485_communication_component(unsigned char device_address, unsigned char& val)
{
    trace( "%s(%i):INFO(3): %s( %u, %u )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_0, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_0, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.component_rs485_0", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_0, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*              0 = Keines
*              1 = MC35i
*              2 = MC55
*            ... = ...
*/
int Datafox::Communication::Module_hardware::read_network_adapter_type( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TCPIP, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TCPIP, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.type_tcpip", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TCPIP, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*              0 = Keines
*              1 = MC35i
*              2 = MC55
*            ... = ...
*/
int Datafox::Communication::Module_hardware::read_modem_type( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MODEM_MC, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MODEM_MC, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.type_modem_mc", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MODEM_MC, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int Datafox::Communication::Module_hardware::read_rs485_access_control_component( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_1, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_1, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.component_rs485_1", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_RS485_1, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*             0 = Keines
*             1 = Lehner kleines
*             2 = Lehner grosses
*           ... = ...
*/
int Datafox::Communication::Module_hardware::read_display_type( unsigned char device_address, unsigned char& val, bool checked )
{
    (void)checked;
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    // Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
    // - Funktionen TypeTransponder liefern <-> Displaywert
    // - Funktionen TypeDisplay liefern <-> Keypadwert
    // - Funktionen TypeKeypad liefern <-> Transponderwert
    //
    // Die Verwechslung wird in der Ableitung dieser Moduleklasse abgefangen.

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DISPLAY, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DISPLAY, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.type_display", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DISPLAY, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*             0 = Keine
*             1 = PZE - Master IV
*             2 = BDE - Master IV
*           ... = ...
*/
int Datafox::Communication::Module_hardware::read_keypad_type( unsigned char device_address, unsigned char& val, bool checked )
{
    (void)checked;
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    // Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
    // - Funktionen TypeTransponder liefern <-> Displaywert
    // - Funktionen TypeDisplay liefern <-> Keypadwert
    // - Funktionen TypeKeypad liefern <-> Transponderwert
    //
    // Die Verwechslung wird in der Ableitung dieser Moduleklasse abgefangen.

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_KEYPAD, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_KEYPAD, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.type_keypad", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_KEYPAD, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*             0 = Keiner
*             1 = TSR30
*             2 = TSR32
*             3 = PHG PRIMO 100
*             4 = Cavitec Mifare
*             5 = Dallas iButton
*           ... = ...
*/
int Datafox::Communication::Module_hardware::read_transponder_reader_type( unsigned char device_address, unsigned char& val, bool checked )
{
    (void)checked;
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    // Impementierungswechsel: Bootloaderversionen 00.04.00.00 und 00.04.00.01 verwalten die Werte anderst:
    // - Funktionen TypeTransponder liefern <-> Displaywert
    // - Funktionen TypeDisplay liefern <-> Keypadwert
    // - Funktionen TypeKeypad liefern <-> Transponderwert
    //
    // Die Verwechslung wird in der Ableitung dieser Moduleklasse abgefangen.

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TRANSPONDER, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TRANSPONDER, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.type_transponder", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TRANSPONDER, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*             0 = Keine
*             1 = Fest integriert
*             2 = Herausnehmbar
*           ... = ...
*/
int Datafox::Communication::Module_hardware::read_mmc_type( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MMC, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MMC, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.type_mmc", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_MMC, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*             0 = Keine
*             1 = Bluegiga (Bluetooth)
*           ... = ...
*/
int Datafox::Communication::Module_hardware::read_radio_type( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_RADIO, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_RADIO, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.type_radio", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_RADIO, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichneten Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*              0 = Kein
*              1 = Furuno GN-80
*            ... = ...
*/
int Datafox::Communication::Module_hardware::read_gps_type( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_GPS, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_GPS, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.type_gps", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_GPS, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int Datafox::Communication::Module_hardware::read_usb_component( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_USB, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_USB, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.component_usb", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_USB, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int Datafox::Communication::Module_hardware::read_battery_component( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_BATTERY, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_BATTERY, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.component_battery", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_BATTERY, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace("%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int Datafox::Communication::Module_hardware::read_battery_charger_component( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_CHARGE, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_CHARGE, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.component_charge", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_CHARGE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*             0 = Keine
*             1 = HKW
*             4 = Nicht zu verwenden, da auf 0 gemappt.
*           ... = ...
*/
int Datafox::Communication::Module_hardware::read_DCF77_type( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DCF77, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DCF77, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        // Wert wurde von FWV 04.01.01 fuer die Baudrate von Bluetooth verwendet und
        // steht nach initialisierung auf dem Wert 4. Dieser Wert, wird als Typ ignoriert.
        val = (m_packet_data[0] != 0xFF && m_packet_data[0] != 4) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.type_dfc77", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_DCF77, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int Datafox::Communication::Module_hardware::read_analog_input_component( unsigned char device_address, unsigned int& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_ANALOG, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_ANALOG, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned int) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (*(unsigned int*)m_packet_data != 0xFFFFFFFF) ? *(unsigned int*)m_packet_data : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned int val = get_property( "hardware_info.component_in_analog", 0 );
        count = sizeof(unsigned int);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_ANALOG, (unsigned char*)&val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int Datafox::Communication::Module_hardware::read_digital_input_component( unsigned char device_address, unsigned int& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_DIGITAL, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_DIGITAL, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned int) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (*(unsigned int*)m_packet_data != 0xFFFFFFFF) ? *(unsigned int*)m_packet_data : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned int val = get_property( "hardware_info.component_in_digital", 0 );
        count = sizeof(unsigned int);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_IN_DIGITAL, (unsigned char*)&val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Wert ob Komponente enthalten = 1 / nicht enthalten = 0
*/
int Datafox::Communication::Module_hardware::read_digital_output_component( unsigned char device_address, unsigned int& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned int) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (*(unsigned int*)m_packet_data != 0xFFFFFFFF) ? *(unsigned int*)m_packet_data : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned int val = get_property( "hardware_info.component_out_digital", 0 );
        count = sizeof(unsigned int);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_COMPONENT_OUT_DIGITAL, (unsigned char*)&val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int Datafox::Communication::Module_hardware::read_io_module_option( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_IOMODULE, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_IOMODULE, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = m_packet_data[0];
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.option_io_module", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_IOMODULE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int Datafox::Communication::Module_hardware::read_com_server_option( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_COMSERVER, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_COMSERVER, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = m_packet_data[0];
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.option_com_server", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_COMSERVER, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace("%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int Datafox::Communication::Module_hardware::read_timeboy_adaption_option( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TBADAPTION, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TBADAPTION, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = m_packet_data[0];
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.option_tb_adaption", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TBADAPTION, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_hardware::inspection_station_command( unsigned char device_address, unsigned char address, char* data, int sizeData, unsigned char& addressAnswer, char* dataAnswer, int& sizeAnswer, int toPending )
{
    trace( "%s(%i):INFO(3): %s( %i, %i, %p, %i )\n", __FILE__, __LINE__, __func__, device_address, address, data, sizeData );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;
    int i;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        // Parameter eintragen
        // Erster Parameter ist die Adresse
        m_packet_data[0] = address;
        count = sizeof(unsigned char);
        // Folgender Parameter ist eine Optionale Zeichenkette
        for ( i = 0; i < sizeData; ++i )
            m_packet_data[count + i] = data[i];

        count += sizeData;
        // Nullterminierung sicherstellen
        if ( m_packet_data[count] != '\0' )
        {
            count++;
            m_packet_data[count] = '\0';
        }

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION, result );
            return result;
        }

        // OK
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Aktive Antwort abwarten
        if ( (result = call( IV_CONTEXT_SLAVE, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION_OK, m_packet_data, count, ((toPending*1000) / get_connection().get_read_timeout()) )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( get_device_address_receiver(), IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION, result );
            return result;
        }

        // Erstes Byte ist die Adresse
        addressAnswer = m_packet_data[0];
        // Folgend ein optionaler String
        for ( i = 0; i < (count-1) && i < sizeAnswer; ++i )
            dataAnswer[i] = m_packet_data[1 + i];

        // Laenge der Zeichenkette
        sizeAnswer = i;
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param device_address Anzusprechende Geraetenummer.
*/
int Datafox::Communication::Module_hardware::reply_inspection_station_command( unsigned char device_address )
{
    trace( "%s(%i):INFO(3): %s( %i )\n", __FILE__, __LINE__, __func__, device_address );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
#ifdef WITH_VIRTUAL_DEVICE
        trace( "%s(%i):ERROR(1): %s()\n", __FILE__, __LINE__, __func__ );
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }
    else
    {
        unsigned char tmp[4];
        count = 0;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_INSPECTION_STATION_OK, tmp, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int Datafox::Communication::Module_hardware::read_transponder_write_option( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TPWRITE, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TPWRITE, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = m_packet_data[0];
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.option_tp_write", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_TPWRITE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int Datafox::Communication::Module_hardware::read_http_option( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_HTTP, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_HTTP, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = m_packet_data[0];
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.option_http", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_HTTP, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int Datafox::Communication::Module_hardware::read_localization_option( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_LANGUAGE, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_LANGUAGE, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = m_packet_data[0];
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.option_language", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_LANGUAGE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt ob die bezeichnete Option enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int Datafox::Communication::Module_hardware::read_fingerprint_scanner_update_option( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_BIOKEY_UPDATE, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_BIOKEY_UPDATE, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = m_packet_data[0];
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.option_biokey_update", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_OPTION_BIOKEY_UPDATE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt die bezahlbaren Optionen enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param mask Maske, welche Optionen unterstuetzt werden.
* @param options Wert ob Option enthalten = 1 / nicht enthalten = 0
*/
int Datafox::Communication::Module_hardware::read_buy_options( unsigned char device_address, unsigned char& mask, unsigned char& options )
{
    trace( "%s(%i):INFO(3): %s( %i, 0x%02X, 0x%02X )\n", __FILE__, __LINE__, __func__, device_address, mask, options );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_PAYABLE_OPTIONS, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_PAYABLE_OPTIONS, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != (2 * sizeof(unsigned char)) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            mask = 0;
            options = 0;
            return result;
        }

        // Werte uebernehmen
        mask = m_packet_data[0];
        options = m_packet_data[1];
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char buf[2];
        buf[0] = get_property( "hardware_info.options_payable_mask", 0 );
        count = sizeof(unsigned char);
        buf[1] = get_property( "hardware_info.options_payable", 0 );
        count += sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_PAYABLE_OPTIONS, buf, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*             0 = Keine
*             1 = Idencom (BioKey 3000)
*           ... = ...
*/
int Datafox::Communication::Module_hardware::read_fingerprint_scanner_type( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_FINGERPRINT, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_FINGERPRINT, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.type_fingerprint", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_FINGERPRINT, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichnete Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Bitfeld der verwendbaren Varianten.
*            -1 = Undefiniert
*            Bit 1 = GIS
*            Bit 2 = Protect
*            Bit 3 = PHG
*           ... = ...
*/
int Datafox::Communication::Module_hardware::read_external_transponder_reader_type( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_EXTERNAL_READER, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_EXTERNAL_READER, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // BUGFIX
        // Eine Firmware die die Serien TS und Protect unterstützt setzt als unterstützte Serien
        // TS und Protect obwohl es so sein sollte, dass die angesprochene Serie eingetragen wird,
        // so dass der Wert der Hardwareinfo eindeutig ist.

        // Wert uebernehmen
        val = m_packet_data[0];

        // Werden beide Flags vorgefunden wird von TS alleine ausgegangen.
        if ( (val != 0xFF) && (val & IV_VALUE_TYPE_EXTERNAL_READER_GIS) && (val & IV_VALUE_TYPE_EXTERNAL_READER_PROTECT) )
        {
            val = IV_VALUE_TYPE_EXTERNAL_READER_GIS;
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.type_external_reader", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_EXTERNAL_READER, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichneten Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*              0 = Kein
*              1 = Exklusiv-Line IV
*              2 = Flex-MasterIV
*            ... = ...
*/
int Datafox::Communication::Module_hardware::read_touchscreen_type( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TOUCH, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TOUCH, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.type_touch", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_TOUCH, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt welcher Typ der bezeichneten Komponente enthalten ist.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Ermittelte Variante.
*              0 = Kein
*              1 = Vibration
*              2 = Beschleunigung
*            ... = ...
*/
int Datafox::Communication::Module_hardware::read_sensor_type( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_SENSOR, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_SENSOR, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.type_sensor", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_TYPE_SENSOR, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Setzen der Optionen fuer Setup.
*
* @param device_address Anzusprechende Geraetenummer.
* @param mask Gibt die gültigen Werte des Wertes val an.
* @param val Gibt an wie die Option gesetzt werden soll.
*         bit 0 = Signalverarbeitung
*           ... = ...
*/
int Datafox::Communication::Module_hardware::write_setup_options( unsigned char device_address, unsigned short mask, unsigned short options )
{
    trace( "%s(%i):INFO(3): %s( %i, 0x%04X, 0x%04X )\n", __FILE__, __LINE__, __func__, device_address, mask, options );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        // Die Komponendenangabe uebernehmen.
        *(unsigned short*)&m_packet_data[0] = mask;
        count += sizeof(unsigned short);

        *(unsigned short*)&m_packet_data[2] = options;
        count += sizeof(unsigned short);

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_SETUP_OPTIONS, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( get_device_address_receiver(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_SETUP_OPTIONS, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        unsigned char buf[4];
        // Parameterwert und -groesse setzen
        *reinterpret_cast<unsigned short*>( &buf[0] ) = get_property( "hardware_info.setup_options_mask", 0 );
        count = sizeof(unsigned short);
        *reinterpret_cast<unsigned short*>( &buf[count] ) = get_property( "hardware_info.setup_options", 0 );
        count += sizeof(unsigned short);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_SETUP_OPTIONS, buf, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermitteln der Optionen fuer Setup.
*
* @param device_address Anzusprechende Geraetenummer.
* @param mask Gibt die gültigen Werte des Wertes val an.
* @param val Gibt an wie die Option gesetzt werden soll.
*         bit 0 = Signalverarbeitung
*           ... = ...
*/
int Datafox::Communication::Module_hardware::read_setup_options( unsigned char device_address, unsigned short& mask, unsigned short& options )
{
    trace( "%s(%i):INFO(3): %s( %i, 0x%04X, 0x%04X )\n", __FILE__, __LINE__, __func__, device_address, mask, options );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_SETUP_OPTIONS, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( get_device_address_receiver(), IV_MODULE_HARDWARE, IV_FUNCTION_GET_SETUP_OPTIONS, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count != (2 * sizeof(unsigned short)) )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }

        // Die Komponendenangabe uebernehmen.
        mask = *(unsigned short*)&m_packet_data[0];
        options = *(unsigned short*)&m_packet_data[2];
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        unsigned char buf[4];
        // Parameterwert und -groesse setzen
        *reinterpret_cast<unsigned short*>( &buf[0] ) = get_property( "hardware_info.type_mmc", 0 );
        count = sizeof(unsigned char);
        *reinterpret_cast<unsigned short*>( &buf[count] ) = get_property( "hardware_info.type_mmc", 0 );
        count += sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_SETUP_OPTIONS, buf, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt die gesamte Hardwareinfo in einer einzelnen Struktur.
*
* @param device_address Anzusprechende Geraetenummer.
* @param ebs Struktur mit Informationen zur Hardware.
*/
int Datafox::Communication::Module_hardware::read_hardware_info( unsigned char device_address, StructHardwareInfo& hwInfo )
{
    trace( "%s(%i):INFO(3): %s( %i )\n", __FILE__, __LINE__, __func__, device_address );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HARDWARE_INFO, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HARDWARE_INFO, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count < (int)sizeof(StructHardwareInfo) )
        {
            // Wenn nur die Bezahloptionen fehlen, dann trotzdem annehmen.
            // Uebergang von 04.02.02 auf 04.02.03 (29.05.2012)
            if (count != 0x2b)
                return 0x0300; // Es wurden weniger Daten gelesen als aktuell bekannt.

            m_packet_data[count - 1] = 0; // Es fehlen noch die Bezahloptionen
            count++;
                
        }
        else if ( count > (int)sizeof(StructHardwareInfo) )
        {
            // Es wurden mehr Daten gelesen als aktuell bekannt.
            return 0x0400;
            // Fehler 4
        }

        // Werte uebernehmen
        hwInfo = *((StructHardwareInfo*)&m_packet_data[0]);
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parametergroesse setzen
        std::fill_n( reinterpret_cast<unsigned char*>( &hwInfo ), sizeof( StructHardwareInfo ), 0 );
        hwInfo.version               = get_property( "hardware_info.version", 0xFF );
        hwInfo.typeDevice            = get_property( "hardware_info.type_device", 0xFF );
        hwInfo.typeBoard             = get_property( "hardware_info.type_board", 0xFF );
        hwInfo.componentRS485_0      = get_property( "hardware_info.component_rs485_0", 0xFF );
        hwInfo.typeTCPIP             = get_property( "hardware_info.type_tcpip", 0xFF );
        hwInfo.typeModemMC           = get_property( "hardware_info.type_modem_mc", 0xFF );
        hwInfo.componentRS485_1      = get_property( "hardware_info.component_rs485_1", 0xFF );
        hwInfo.typeTransponder       = get_property( "hardware_info.type_transponder", 0xFF );
        hwInfo.typeDisplay           = get_property( "hardware_info.type_display", 0xFF );
        hwInfo.typeKeypad            = get_property( "hardware_info.type_keypad", 0xFF );
        hwInfo.typeMMC               = get_property( "hardware_info.type_mmc", 0xFF );
        hwInfo.typeGPS               = get_property( "hardware_info.type_gps", 0xFF );
        hwInfo.componentUSB          = get_property( "hardware_info.component_usb", 0xFF );
        hwInfo.typeRadio             = get_property( "hardware_info.type_radio", 0xFF );
        hwInfo.typeDCF77             = get_property( "hardware_info.type_dfc77", 0xFF );
        hwInfo.componentInAnalog     = get_property( "hardware_info.component_in_analog", 0xFFFFFFFF );
        hwInfo.componentInDigital    = get_property( "hardware_info.component_in_digital", 0xFFFFFFFF );
        hwInfo.componentOutDigital   = get_property( "hardware_info.component_out_digital", 0xFFFFFFFF );
        hwInfo.typeFingerprint       = get_property( "hardware_info.type_fingerprint", 0xFF );
        hwInfo.typeExternalReader    = get_property( "hardware_info.type_external_reader", 0xFF );
        hwInfo.optionIOModule        = get_property( "hardware_info.option_io_module", 0xFF );
        hwInfo.optionTBAdaption      = get_property( "hardware_info.option_tb_adaption", 0xFF );
        hwInfo.optionComserver       = get_property( "hardware_info.option_com_server", 0xFF );
        hwInfo.optionTPWrite         = get_property( "hardware_info.option_tp_write", 0xFF );
        hwInfo.optionHttp            = get_property( "hardware_info.option_http", 0xFF );
        hwInfo.optionLanguage        = get_property( "hardware_info.option_language", 0xFF );
        hwInfo.optionBiokeyUpdate    = get_property( "hardware_info.option_biokey_update", 0xFF );
        hwInfo.optionsSetup          = get_property( "hardware_info.option_setup", 0xFFFF );
        hwInfo.typeTouch             = get_property( "hardware_info.type_touch", 0xFF );
        hwInfo.typeSensor            = get_property( "hardware_info.type_sensor", 0xFF );
        hwInfo.componentBattery      = get_property( "hardware_info.component_battery", 0xFF );
        hwInfo.componentCharge       = get_property( "hardware_info.component_charge", 0xFF );
        hwInfo.optionsPayable        = get_property( "hardware_info.option_payable", 0xFF );
        count = sizeof(StructHardwareInfo);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HARDWARE_INFO,
                             reinterpret_cast<unsigned char*>( &hwInfo ), count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

// ##################################################################
/**
* @brief Ermittelt die "Version" der Hardwareconfiguration.
*
* @param device_address Anzusprechende Geraetenummer.
* @param val Wert fuer Version der Hardwareconfiguration.
*/
int Datafox::Communication::Module_hardware::read_hardware_info_version( unsigned char device_address, unsigned char& val )
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HW_CONFIG, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( device_address, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HW_CONFIG, result );
            return result;
        }

        // Antwortparameter pruefen
        if ( count != sizeof(unsigned char) )
        {
            trace( "%s(%i):WARNING(1): %s() [ Antwortparameter fehlt ]\n", __FILE__, __LINE__, __func__, count );
            val = 0;
            return result;
        }

        // Wert uebernehmen
        val = (m_packet_data[0] != 0xFF) ? m_packet_data[0] : 0;
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        int result;
        // Parameterwert und -groesse setzen
        unsigned char val = get_property( "hardware_info.version", 0 );
        count = sizeof(unsigned char);

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_GET_HW_CONFIG, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}


#if !defined( DATAFOX_STUDIO_IV ) && !defined( DF_API_PRODUCTION ) && !defined( WITH_VIRTUAL_DEVICE )
// Funktions-Stumps

int Datafox::Communication::Module_hardware::random_ping( unsigned char device_address )
{ 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;

    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_device_type( unsigned char device_address, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_board_type( unsigned char device_address, unsigned char val, unsigned short val2 )
{ 
    (void)val;
    (void)val2;
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_network_adapter_type( unsigned char device_address, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_modem_type( unsigned char device_address, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_display_type( unsigned char device_address, unsigned char val, unsigned short val2, bool checked )
{ 
    (void)val; 
    (void)val2; 
    (void)checked; 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_keypad_type( unsigned char device_address, unsigned char val, unsigned short val2, bool checked )
{ 
    (void)val; 
    (void)val2; 
    (void)checked; 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_transponder_reader_type( unsigned char device_address, unsigned char val, unsigned short val2, bool checked )
{ 
    (void)val; 
    (void)val2; 
    (void)checked; 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_mmc_type( unsigned char device_address, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_radio_type( unsigned char device_address, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_gps_type( unsigned char device_address, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_DCF77_type( unsigned char device_address, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_fingerprint_scanner_type( unsigned char device_address, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_external_transponder_reader_type( unsigned char device_address, unsigned char val, unsigned short val2 ) 
{
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    unsigned short key;
    switch ( val ) 
    {
    case 0xFF:
        key = 0xCF38;
        break;

    case 0x00:
        key = 0x8F78;
        break;

    case 0x01:
        key = 0x4FB9;
        break;

    case 0x02:
        key = 0x4EF9;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        // Die Komponendenangabe uebernehmen.
        m_packet_data[0] = val;
        count += sizeof(unsigned char);

        *(unsigned short*)&m_packet_data[1] = key;
        count += sizeof(unsigned short);

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_TYPE_EXTERNAL_READER, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( get_device_address_receiver(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_TYPE_EXTERNAL_READER, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        // Schluessel pruefen: Funktionsfehler
        if ( key != val2 )
            return 0x0600;

        int result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        m_hardware_info.optionBiokeyUpdate = val;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_TYPE_EXTERNAL_READER, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}
int Datafox::Communication::Module_hardware::write_touchscreen_type( unsigned char device_address, unsigned char val, unsigned short val2 )
{ 
    (void)val;
    (void)val2;
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_sensor_type( unsigned char device_address, unsigned char val, unsigned short val2 )
{ 
    (void)val;
    (void)val2;
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}


int Datafox::Communication::Module_hardware::write_rs485_communication_component( unsigned char device_address, unsigned char val, unsigned short val2 )
{ 
    (void)val;
    (void)val2;
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_rs485_access_control_component( unsigned char device_address, unsigned char val, unsigned short val2 )
{ 
    (void)val;
    (void)val2;
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_usb_component( unsigned char device_address, unsigned char val, unsigned short val2 )
{ 
    (void)val;
    (void)val2;
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_analog_input_component( unsigned char device_address, unsigned int val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_digital_input_component( unsigned char device_address, unsigned int val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_digital_output_component( unsigned char device_address, unsigned int val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_battery_component( unsigned char device_address, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_battery_charger_component( unsigned char device_address, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE; 
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_io_module_option( unsigned char device_address, unsigned char val, unsigned short val2 )     
{ 
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    unsigned short key;
    switch ( val )
    {
    case 0xFF:
        key = 0x0C59;
        break;

    case 0x00:
        key = 0x4C19;
        break;

    case 0x01:
        key = 0x8CD8;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        // Die Komponendenangabe uebernehmen.
        m_packet_data[0] = val;
        count += sizeof(unsigned char);

        *(unsigned short*)&m_packet_data[1] = key;
        count += sizeof(unsigned short);

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_IOMODULE, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( get_device_address_receiver(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_IOMODULE, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        // Schluessel pruefen: Funktionsfehler
        if ( key != val2 )
            return 0x0600;

        int result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        m_hardware_info.optionIOModule = val;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_IOMODULE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

int Datafox::Communication::Module_hardware::write_com_server_option( unsigned char device_address, unsigned char val, unsigned short val2 )    
{ 
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    unsigned short key;
    switch ( val )
    {
    case 0xFF:
        key = 0xCCF8;
        break;

    case 0x00:
        key = 0x8CB8;
        break;

    case 0x01:
        key = 0x4C79;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        // Die Komponendenangabe uebernehmen.
        m_packet_data[0] = val;
        count += sizeof(unsigned char);

        *(unsigned short*)&m_packet_data[1] = key;
        count += sizeof(unsigned short);

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_COMSERVER, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( get_device_address_receiver(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_COMSERVER, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else
    {
#ifdef WITH_VIRTUAL_DEVICE
        // Schluessel pruefen: Funktionsfehler
        if ( key != val2 )
            return 0x0600;

        int result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        m_hardware_info.optionComserver = val;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_COMSERVER, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

int Datafox::Communication::Module_hardware::write_timeboy_adaption_option( unsigned char device_address, unsigned char val, unsigned short val2 )   
{ 
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    unsigned short key;
    switch ( val )
    {
    case 0xFF:
        key = 0xCD18;
        break;

    case 0x00:
        key = 0x8D58;
        break;

    case 0x01:
        key = 0x4D99;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        // Die Komponendenangabe uebernehmen.
        m_packet_data[0] = val;
        count += sizeof(unsigned char);

        *(unsigned short*)&m_packet_data[1] = key;
        count += sizeof(unsigned short);

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TBADAPTION, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( get_device_address_receiver(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TBADAPTION, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        // Schluessel pruefen: Funktionsfehler
        if ( key != val2 )
            return 0x0600;

        int result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        m_hardware_info.optionTBAdaption = val;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TBADAPTION, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

int Datafox::Communication::Module_hardware::write_transponder_write_option( unsigned char device_address, unsigned char val, unsigned short val2 )   
{ 
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    unsigned short key;
    switch ( val )
    {
    case 0xFF:
        key = 0x0BD9;
        break;

    case 0x00:
        key = 0x4B99;
        break;

    case 0x01:
        key = 0x8B58;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }
    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        // Die Komponendenangabe uebernehmen.
        m_packet_data[0] = val;
        count += sizeof(unsigned char);

        *(unsigned short*)&m_packet_data[1] = key;
        count += sizeof(unsigned short);

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TPWRITE, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( get_device_address_receiver(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TPWRITE, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        // Schluessel pruefen: Funktionsfehler
        if (key != val2) return 0x0600;

        int result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        m_hardware_info.optionTPWrite = val;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_TPWRITE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

int Datafox::Communication::Module_hardware::write_http_option( unsigned char device_address, unsigned char val, unsigned short val2  )   
{ 
    trace("%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val);

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    unsigned short key;
    switch ( val )
    {
    case 0xFF:
        key = 0xCB78;
        break;

    case 0x00:
        key = 0x8B38;
        break;

    case 0x01:
        key = 0x4BF9;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        // Die Komponendenangabe uebernehmen.
        m_packet_data[0] = val;
        count += sizeof(unsigned char);

        *(unsigned short*)&m_packet_data[1] = key;
        count += sizeof(unsigned short);

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_HTTP, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( get_device_address_receiver(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_HTTP, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        // Schluessel pruefen: Funktionsfehler
        if (key != val2) return 0x0600;

        int result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        m_hardware_info.optionHttp = val;

        // Antwort des Befehls senden.
        if ((result = call(IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_HTTP, &val, count)) != IV_ERROR_SUCCESS)
        {
            trace( "%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result );
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}

int Datafox::Communication::Module_hardware::write_localization_option( unsigned char device_address, unsigned char val, unsigned short val2 )
{ 
    (void)val; 
    (void)val2; 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_fingerprint_scanner_update_option( unsigned char device_address, unsigned char val, unsigned short val2 )
{ 
    trace( "%s(%i):INFO(3): %s( %i, %i )\n", __FILE__, __LINE__, __func__, device_address, val );

    int result = IV_ERROR_SUCCESS;
    uint16_t count = 0;

    unsigned short key;
    switch ( val )
    {
    case 0xFF:
        key = 0x0A39;
        break;

    case 0x00:
        key = 0x4A79;
        break;

    case 0x01:
        key = 0x8AB8;
        break;

    default:
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    // Funktion in Remoteklasse aufrufen?
    if ( !is_own_device_address( device_address ) )
    {
        // Befehlsausfuehrung geschuetzt durch Mutex.
        lock_guard_call guard( m_mutex_call );

        // Umgebung zuruecksetzen
        reset_error();

        // Remoteadresse setzen.
        set_device_address_receiver( device_address );

        // Die Komponendenangabe uebernehmen.
        m_packet_data[0] = val;
        count += sizeof(unsigned char);

        *(unsigned short*)&m_packet_data[1] = key;
        count += sizeof(unsigned short);

        if ( (result = call( IV_CONTEXT_MASTER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_BIOKEY_UPDATE, m_packet_data, count )) != IV_ERROR_SUCCESS )
        {
            // Fehler ausgeben
            trace_error( get_device_address_receiver(), IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_BIOKEY_UPDATE, result );
            return result;
        }

        // Keine Antwortdaten erwartet
        if ( count )
        {
            trace( "%s(%i):WARNING(3): %s() [ Antwort enthaelt %i Byte an unnuetze Daten. ]\n", __FILE__, __LINE__, __func__, count );
        }
    }
    else 
    {
#ifdef WITH_VIRTUAL_DEVICE
        // Schluessel pruefen: Funktionsfehler
        if ( key != val2 )
            return 0x0600;

        int result;
        // Parametergroesse setzen
        count = 0;

        // Wert an interne Struktur uebernehmen
        m_hardware_info.optionBiokeyUpdate = val;

        // Antwort des Befehls senden.
        if ( (result = call( IV_CONTEXT_TRANSMIT_ANSWER, IV_MODULE_HARDWARE, IV_FUNCTION_SET_OPTION_BIOKEY_UPDATE, &val, count )) != IV_ERROR_SUCCESS )
        {
            trace("%s(%i):ERROR(1): %s() [ call (%i) ]\n", __FILE__, __LINE__, __func__, result);
            return result;
        }
#else
        result = 0x0200; // Funktion nicht verfügbar.
#endif
    }

    return result;
}
int Datafox::Communication::Module_hardware::write_buy_options( unsigned char device_address, unsigned char mask, unsigned char options, unsigned short val2 )
{ 
    (void)mask; 
    (void)options; 
    (void)val2; 
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

int Datafox::Communication::Module_hardware::write_hardware_info_version( unsigned char device_address, unsigned char val, unsigned short val2 )
{
    (void)val;
    (void)val2;
    if ( !is_own_device_address( device_address ) )
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    
    return 0x0200;
}

#endif
